Imports System.Net.Sockets
Imports System.IO
Public Class Client : Implements IDisposable

   Private _TcpClient As TcpClient, _Reader As BinaryReader, _Writer As BinaryWriter, _dlgReadString As Func(Of String)

   Public Sub New(ByVal TC As TcpClient)
      _TcpClient = TC
      Dim strm = _TcpClient.GetStream
      _Reader = New BinaryReader(strm)
      _Writer = New BinaryWriter(strm)
      _dlgReadString = AddressOf _Reader.ReadString
      _dlgReadString.BeginInvoke(AddressOf EndRead, Nothing)
   End Sub

   Private Sub EndRead(ByVal ar As IAsyncResult)
      If IsDisposed Then Return
      Try
         Dim msg = _dlgReadString.EndInvoke(ar)
         CrossThread.RunGui(AddressOf OnChatMessage, msg)
         _dlgReadString.BeginInvoke(AddressOf EndRead, Nothing) 'asynchrone Rekursion
      Catch ex As EndOfStreamException
         CrossThread.RunGui(AddressOf OnStatusMessage, "CounterClient shut down")
         CrossThread.RunGui(AddressOf Dispose)
      End Try
   End Sub

   Public Sub Send(ByVal Msg As String)
      _Writer.Write(Msg)
   End Sub

   Protected Sub Dispose(ByVal disposing As Boolean)
      DisposeAll(_Writer, _Reader, _TcpClient)
   End Sub

   'TcpBase
   Private _IsDisposed As Boolean = False
   Public Event Disposed As EventHandlerEx(Of Client)
   Public Sub Dispose() Implements IDisposable.Dispose
      If _IsDisposed Then Return
      _IsDisposed = True
      Dispose(True)           ' rufe die erzwungenen berschreibungen von Sub Dispose(Boolean)
      OnStatusMessage(Me.GetType.Name, " disposed")
      RaiseEvent Disposed(Me)
      GC.SuppressFinalize(Me)
   End Sub

   ''' <summary>
   ''' Zur Ausgabe chat-verwaltungstechnischer Status-Informationen
   ''' </summary>
   Public Event StatusMessage As EventHandler(Of NotifyEventArgs(Of String))
   Protected Sub OnStatusMessage(ByVal ParamArray segments As String())
      RaiseEvent StatusMessage(Me, New NotifyEventArgs(Of String)(String.Concat(segments)))
   End Sub

   ''' <summary>Zur Ausgabe von Chat-Messages</summary>
   Public Event ChatMessage As EventHandler(Of NotifyEventArgs(Of String))
   Protected Sub OnChatMessage(ByVal ParamArray segments As String())
      RaiseEvent ChatMessage(Me, New NotifyEventArgs(Of String)(String.Concat(segments)))
   End Sub

   Public ReadOnly Property IsDisposed() As Boolean
      Get
         Return _IsDisposed
      End Get
   End Property

End Class

#Region "Fail mit Ssl wg. Certificate-Theater"
#If False Then
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Net.Security
Public Class Client : Inherits TCPBase
   Private _TcpClient As TcpClient
   Private _DataAvailable As Func(Of Boolean)
   Private _Stream As SslStream
   Private _Buf(&H400 - 1) As Byte
   Public Sub New( TC As TcpClient)
      _TcpClient = TC
      Dim strm = _TcpClient.GetStream
      _DataAvailable = Function() strm.DataAvailable
      _Stream = New SslStream(strm, False, AddressOf RemoteCertificateValidationCallback)
      _Stream.BeginRead(_Buf, 0, _Buf.Length, AddressOf EndRead, Nothing)
   End Sub
   Function RemoteCertificateValidationCallback( sender As Object,  certificate As System.Security.Cryptography.X509Certificates.X509Certificate,  chain As System.Security.Cryptography.X509Certificates.X509Chain,  sslPolicyErrors As System.Net.Security.SslPolicyErrors) As Boolean
      Return True
   End Function
   Private Sub EndRead( ar As IAsyncResult)
      If MyBase.IsDisposed Then Return
      Dim read As Integer = _Stream.EndRead(ar)
      If read = 0 Then 'leere Datenbermittlung signalisiert Verbindungsabbruch
         CrossThread.RunGui(AddressOf OnStatusMessage, _
            New NotifyEventArgs(Of String)("CounterClient shut down"))
         CrossThread.RunGui(AddressOf MyBase.Dispose)
         Return
      End If
      Dim SB As New StringBuilder(Encoding.UTF8.GetString(_Buf, 0, read))
      Do While _DataAvailable()
         read = _Stream.Read(_Buf, 0, _Buf.Length)
         SB.Append(Encoding.UTF8.GetString(_Buf, 0, read))
      Loop
      CrossThread.RunGui(AddressOf OnChatMessage, New NotifyEventArgs(Of String)(SB.ToString))
      _Stream.BeginRead(_Buf, 0, _Buf.Length, AddressOf EndRead, Nothing)
   End Sub
   Public Overrides Sub Send( Msg As String)
      Dim Buf() As Byte = Encoding.UTF8.GetBytes(Msg)
      _Stream.Write(Buf, 0, Buf.Length)
   End Sub
   Protected Overrides Sub Dispose( disposing As Boolean)
      DisposeAll(_Stream, _TcpClient)
   End Sub
End Class
#End If
#End Region 'Fail mit Ssl wg. Certificate-Theater