Imports System.Net
Imports System.Net.Sockets

Public Class Server : Implements IDisposable

   Private _Listener As TcpListener

   'Pro Verbindung(-sanfrage) wird ein Client-Objekt generiert, das den Datenaustausch dieser Verbindung abwickelt
   Private _Clients As New List(Of Client)

   Public Sub New(ByVal EP As IPEndPoint)
      _Listener = New TcpListener(EP)
      _Listener.ExclusiveAddressUse = False
      _Listener.Start()
      _Listener.BeginAcceptTcpClient(AddressOf EndAccept, Nothing)
   End Sub

   Private Sub EndAccept(ByVal ar As IAsyncResult)
      If IsDisposed Then Return
      Dim clnt=New Client(_Listener.EndAcceptTcpClient(ar))
      AddHandler clnt.ChatMessage, AddressOf Client_ChatMessage
      AddHandler clnt.StatusMessage, AddressOf Client_StatusMessage
      AddHandler clnt.Disposed, AddressOf Client_Disposed
      _Clients.Add(clnt)
      CrossThread.RunGui(AddressOf OnStatusMessage, "TCPClient accepted")
      _Listener.BeginAcceptTcpClient(AddressOf EndAccept, Nothing)
   End Sub

#Region "_Clients-Ereignisverarbeitung"

   Private Sub Client_Disposed(ByVal Sender As Client)
      'den Client fr die beendete Verbindung entfernen
      _Clients.Remove(Sender)
   End Sub

   Private Sub Client_ChatMessage(ByVal sender As Object, ByVal e As NotifyEventArgs(Of String))
      'einkommende ChatMessages anzeigen, und an alle versenden
      Send(e.Value)
   End Sub

   Private Sub Client_StatusMessage(ByVal sender As Object, ByVal e As NotifyEventArgs(Of String))
      'einkommende StatusMessages durchreichen (zur Anzeige)
      OnStatusMessage(e.Value)
   End Sub

#End Region '_Clients-Ereignisverarbeitung

   Public Sub Send(ByVal Msg As String)
      OnChatMessage(Msg) ' anzeigen
      For Each C As Client In _Clients    ' an alle versenden
         C.Send(Msg)
      Next
   End Sub

   Protected Sub Dispose(ByVal disposing As Boolean)
      _Listener.Stop()
      For i As Integer = _Clients.Count - 1 To 0 Step -1
         _Clients(i).Dispose()
      Next
   End Sub
   'TcpBase
   Private _IsDisposed As Boolean = False
   Public Event Disposed As EventHandlerEx(Of Server)
   Public Sub Dispose() Implements IDisposable.Dispose
      If _IsDisposed Then Return
      _IsDisposed = True
      Dispose(True)           ' rufe die erzwungenen berschreibungen von Sub Dispose(Boolean)
      OnStatusMessage(Me.GetType.Name, " disposed")
      RaiseEvent Disposed(Me)
      GC.SuppressFinalize(Me)
   End Sub

   ''' <summary>
   ''' Zur Ausgabe chat-verwaltungstechnischer Status-Informationen
   ''' </summary>
   Public Event StatusMessage As EventHandler(Of NotifyEventArgs(Of String))
   Protected Sub OnStatusMessage(ByVal ParamArray segments As String())
      RaiseEvent StatusMessage(Me, New NotifyEventArgs(Of String)(String.Concat(segments)))
   End Sub

   ''' <summary>Zur Ausgabe von Chat-Messages</summary>
   Public Event ChatMessage As EventHandler(Of NotifyEventArgs(Of String))
   Protected Sub OnChatMessage(ByVal ParamArray segments As String())
      RaiseEvent ChatMessage(Me, New NotifyEventArgs(Of String)(String.Concat(segments)))
   End Sub

   Public ReadOnly Property IsDisposed() As Boolean
      Get
         Return _IsDisposed
      End Get
   End Property


End Class
